"use strict";

const codeSign_1 = require("./codeSign");
const bluebird_1 = require("bluebird");
const platformPackager_1 = require("./platformPackager");
const metadata_1 = require("./metadata");
const path = require("path");
const log_1 = require("./util/log");
const fs_extra_p_1 = require("fs-extra-p");
const signcode_tf_1 = require("signcode-tf");
const squirrelWindows_1 = require("./targets/squirrelWindows");
const nsis_1 = require("./targets/nsis");
const targetFactory_1 = require("./targets/targetFactory");
//noinspection JSUnusedLocalSymbols
const __awaiter = require("./util/awaiter");
class WinPackager extends platformPackager_1.PlatformPackager {
    constructor(info, cleanupTasks) {
        super(info);
        const certificateFile = this.platformSpecificBuildOptions.certificateFile;
        if (certificateFile != null) {
            const certificatePassword = this.platformSpecificBuildOptions.certificatePassword || this.getCscPassword();
            this.cscInfo = bluebird_1.Promise.resolve({
                file: certificateFile,
                password: certificatePassword == null ? null : certificatePassword.trim()
            });
        } else if (this.options.cscLink != null) {
            this.cscInfo = codeSign_1.downloadCertificate(this.options.cscLink).then(path => {
                cleanupTasks.push(() => fs_extra_p_1.deleteFile(path, true));
                return {
                    file: path,
                    password: this.getCscPassword()
                };
            });
        } else {
            this.cscInfo = bluebird_1.Promise.resolve(null);
        }
        this.iconPath = this.getValidIconPath();
    }
    createTargets(targets, mapper, cleanupTasks) {
        for (let name of targets) {
            if (name === targetFactory_1.DIR_TARGET) {
                continue;
            }
            if (name === targetFactory_1.DEFAULT_TARGET || name === "squirrel") {
                mapper("squirrel", () => {
                    const targetClass = require("./targets/squirrelWindows").default;
                    return new targetClass(this);
                });
            } else if (name === "nsis") {
                mapper(name, outDir => {
                    const targetClass = require("./targets/nsis").default;
                    return new targetClass(this, outDir);
                });
            } else {
                mapper(name, () => targetFactory_1.createCommonTarget(name));
            }
        }
    }
    get platform() {
        return metadata_1.Platform.WINDOWS;
    }
    getIconPath() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.iconPath;
        });
    }
    getValidIconPath() {
        return __awaiter(this, void 0, void 0, function* () {
            let iconPath = this.platformSpecificBuildOptions.icon || this.devMetadata.build.icon;
            if (iconPath != null && !iconPath.endsWith(".ico")) {
                iconPath += ".ico";
            }
            iconPath = iconPath == null ? yield this.getDefaultIcon("ico") : path.resolve(this.projectDir, iconPath);
            if (iconPath == null) {
                return null;
            }
            yield checkIcon(iconPath);
            return iconPath;
        });
    }
    pack(outDir, arch, targets, postAsyncTasks) {
        return __awaiter(this, void 0, void 0, function* () {
            const packOptions = yield this.computePackOptions();
            const appOutDir = this.computeAppOutDir(outDir, arch);
            yield this.doPack(packOptions, outDir, appOutDir, this.platform.nodeName, arch, this.platformSpecificBuildOptions);
            yield this.sign(path.join(appOutDir, `${ this.appInfo.productFilename }.exe`));
            this.packageInDistributableFormat(outDir, appOutDir, arch, targets, postAsyncTasks);
        });
    }
    computeAppOutDir(outDir, arch) {
        return path.join(outDir, `win${ platformPackager_1.getArchSuffix(arch) }-unpacked`);
    }
    sign(file) {
        return __awaiter(this, void 0, void 0, function* () {
            const cscInfo = yield this.cscInfo;
            if (cscInfo != null) {
                log_1.log(`Signing ${ path.basename(file) } (certificate file "${ cscInfo.file }")`);
                yield this.doSign({
                    path: file,
                    cert: cscInfo.file,
                    password: cscInfo.password,
                    name: this.appInfo.productName,
                    site: yield this.appInfo.computePackageUrl(),
                    overwrite: true,
                    hash: this.platformSpecificBuildOptions.signingHashAlgorithms
                });
            }
        });
    }
    //noinspection JSMethodCanBeStatic
    doSign(opts) {
        return __awaiter(this, void 0, void 0, function* () {
            return bluebird_1.Promise.promisify(signcode_tf_1.sign)(opts);
        });
    }
    packageInDistributableFormat(outDir, appOutDir, arch, targets, promises) {
        for (let target of targets) {
            if (target instanceof squirrelWindows_1.default) {
                promises.push(log_1.task(`Building Squirrel.Windows installer`, target.build(arch, appOutDir)));
            } else if (target instanceof nsis_1.default) {
                promises.push(target.build(arch, appOutDir));
            } else {
                const format = target.name;
                log_1.log(`Creating Windows ${ format }`);
                // we use app name here - see https://github.com/electron-userland/electron-builder/pull/204
                const outFile = path.join(outDir, this.generateName1(format, arch, "win", false));
                promises.push(this.archiveApp(format, appOutDir, outFile).then(() => this.dispatchArtifactCreated(outFile, this.generateName1(format, arch, "win", true))));
            }
        }
    }
}
exports.WinPackager = WinPackager;
function checkIcon(file) {
    return __awaiter(this, void 0, void 0, function* () {
        const fd = yield fs_extra_p_1.open(file, "r");
        const buffer = new Buffer(512);
        try {
            yield fs_extra_p_1.read(fd, buffer, 0, buffer.length, 0);
        } finally {
            yield fs_extra_p_1.close(fd);
        }
        if (!isIco(buffer)) {
            throw new Error(`Windows icon is not valid ico file, please fix "${ file }"`);
        }
        const sizes = parseIco(buffer);
        for (let size of sizes) {
            if (size.w >= 256 && size.h >= 256) {
                return;
            }
        }
        throw new Error(`Windows icon size must be at least 256x256, please fix "${ file }"`);
    });
}
function parseIco(buffer) {
    if (!isIco(buffer)) {
        throw new Error("buffer is not ico");
    }
    const n = buffer.readUInt16LE(4);
    const result = new Array(n);
    for (let i = 0; i < n; i++) {
        result[i] = {
            w: buffer.readUInt8(6 + i * 16) || 256,
            h: buffer.readUInt8(7 + i * 16) || 256
        };
    }
    return result;
}
function isIco(buffer) {
    return buffer.readUInt16LE(0) === 0 && buffer.readUInt16LE(2) === 1;
}
//# sourceMappingURL=winPackager.js.map